@@ -93,15 +93,14 @@ void jack_host::create()
 }
 
 void jack_host::create_ports() {
-    char buf[32];
+    char buf[64];
     char buf2[64];
     string prefix = client->name + ":";
-    static const char *suffixes[] = { "#1", "#2", "#3", "#4", "#5", "#6", "#7", "#8" };
     port *inputs = get_inputs();
     port *outputs = get_outputs();
     int in_count = metadata->get_input_count(), out_count = metadata->get_output_count();
     for (int i=0; i<in_count; i++) {
-        snprintf(buf, sizeof(buf), "%s In %s", instance_name.c_str(), suffixes[i]);
+        snprintf(buf, sizeof(buf), "%s In #%d", instance_name.c_str(), i+1);
         snprintf(buf2, sizeof(buf2), client->input_name.c_str(), client->input_nr++);
         inputs[i].nice_name = buf;
         inputs[i].name = buf2;
@@ -123,7 +122,7 @@ void jack_host::create_ports() {
         jack_port_set_alias(midi_port.handle, (prefix + buf2).c_str());
     }
     for (int i=0; i<out_count; i++) {
-        snprintf(buf, sizeof(buf), "%s Out %s", instance_name.c_str(), suffixes[i]);
+        snprintf(buf, sizeof(buf), "%s Out #%d", instance_name.c_str(), i+1);
         snprintf(buf2, sizeof(buf2), client->output_name.c_str(), client->output_nr++);
         outputs[i].nice_name = buf;
         outputs[i].name = buf2;
@@ -135,6 +134,42 @@ void jack_host::create_ports() {
     }
 }
 
+#if JACK_HAS_RENAME
+#define jack_port_rename_fn jack_port_rename
+#else
+#define jack_port_rename_fn(client, handle, name) jack_port_set_name(handle, name)
+#endif
+
+void jack_host::rename_ports() {
+    char buf[64];
+    port *inputs = get_inputs();
+    port *outputs = get_outputs();
+    int in_count = metadata->get_input_count(), out_count = metadata->get_output_count();
+    for (int i=0; i<in_count; i++) {
+        snprintf(buf, sizeof(buf), "%s In #%d", instance_name.c_str(), i+1);
+        inputs[i].nice_name = buf;
+        jack_port_rename_fn(client->client, inputs[i].handle, buf);
+    }
+    if (metadata->get_midi()) {
+        snprintf(buf, sizeof(buf), "%s MIDI In", instance_name.c_str());
+        midi_port.nice_name = buf;
+        jack_port_rename_fn(client->client, midi_port.handle, buf);
+    }
+    for (int i=0; i<out_count; i++) {
+        snprintf(buf, sizeof(buf), "%s Out #%d", instance_name.c_str(), i+1);
+        outputs[i].nice_name = buf;
+        jack_port_rename_fn(client->client, outputs[i].handle, buf);
+        // replacement:
+        // jack_port_rename(client->client, (outputs[i].handle, buf);
+        // but we have to check which jack version first
+    }
+}
+
+void jack_host::rename(std::string name) {
+    instance_name = name;
+    rename_ports();
+}
+
 void jack_host::handle_automation_cc(uint32_t designator, int value)
 {
     last_designator = designator;
@@ -159,7 +194,7 @@ uint32_t jack_host::get_last_automation_source()
 
 void jack_host::handle_event(uint8_t *buffer, uint32_t size)
 {
-    int channel = buffer[0] & 15;
+    int channel = (buffer[0] & 0x0f) + 1;
     int value;
     switch(buffer[0] >> 4)
     {
@@ -382,7 +417,7 @@ char *jack_host::configure(const char *key, const char *value)
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-static const char *short_options = "c:i:l:o:m:M:s:S:ehv";
+static const char *short_options = "c:i:l:o:m:M:s:S:ehvL";
 
 static struct option long_options[] = {
     {"help", 0, 0, 'h'},
@@ -392,9 +427,11 @@ static struct option long_options[] = {
     {"load", 1, 0, 'l'},
     {"input", 1, 0, 'i'},
     {"output", 1, 0, 'o'},
+    {"midi", 1, 0, 'm'},
     {"state", 1, 0, 's'},
     {"connect-midi", 1, 0, 'M'},
     {"session-id", 1, 0, 'S'},
+    {"list", 0, 0, 'L'},
     {0,0,0,0},
 };
 
@@ -402,7 +439,7 @@ void print_help(char *argv[])
 {
     printf("JACK host for Calf effects\n"
         "Syntax: %s [--client <name>] [--input <name>] [--output <name>] [--midi <name>] [--load|state <session>]\n"
-        "       [--connect-midi <name|capture-index>] [--help] [--version] [!] pluginname[:<preset>] [!] ...\n", 
+        "       [--connect-midi <name|capture-index>] [--help] [--version] [--list] [!] pluginname[:<preset>] [!] ...\n", 
         argv[0]);
 }
 
@@ -497,6 +534,15 @@ int main(int argc, char *argv[])
                 else
                     sess.autoconnect_midi = string(optarg);
                 break;
+            case 'L':
+                string s = 
+                #define PER_MODULE_ITEM(name, isSynth, jackname) jackname " "
+                #include <calf/modulelist.h>
+                ;
+                if (!s.empty())
+                    s = s.substr(0, s.length() - 1);
+                printf("%s\n", s.c_str());
+                return 0;
         }
     }
     while(optind < argc) {
