@@ -96,6 +96,7 @@ void gain_reduction_audio_module::process(float &left, float &right, const float
     if(!det_right) {
         det_right = &right;
     }
+    float gain = 1.f;
     if(bypass < 0.5f) {
         // this routine is mainly copied from thor's compressor module
         // greatest sounding compressor I've heard!
@@ -110,11 +111,10 @@ void gain_reduction_audio_module::process(float &left, float &right, const float
         dsp::sanitize(linSlope);
 
         linSlope += (absample - linSlope) * (absample > linSlope ? attack_coeff : release_coeff);
-        float gain = 1.f;
+        
         if(linSlope > 0.f) {
             gain = output_gain(linSlope, rms);
         }
-
         left *= gain * makeup;
         right *= gain * makeup;
         meter_out = std::max(fabs(left), fabs(right));;
@@ -195,7 +195,7 @@ float gain_reduction_audio_module::get_comp_level() {
     return meter_comp;
 }
 
-bool gain_reduction_audio_module::get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const
+bool gain_reduction_audio_module::_get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const
 {
     redraw_graph = false;
     if (!is_active or subindex > 1)
@@ -224,7 +224,7 @@ bool gain_reduction_audio_module::get_graph(int subindex, float *data, int point
     return true;
 }
 
-bool gain_reduction_audio_module::get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const
+bool gain_reduction_audio_module::_get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const
 {
     if (!is_active or bypass > 0.5f or mute > 0.f or subindex)
         return false;
@@ -236,7 +236,7 @@ bool gain_reduction_audio_module::get_dot(int subindex, float &x, float &y, int
     return true;
 }
 
-bool gain_reduction_audio_module::get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+bool gain_reduction_audio_module::_get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
 {
     if (!is_active)
         return false;
@@ -257,7 +257,7 @@ bool gain_reduction_audio_module::get_gridline(int subindex, float &pos, bool &v
     return result;
 }
 
-bool gain_reduction_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+bool gain_reduction_audio_module::_get_layers(int index, int generation, unsigned int &layers) const
 {
     layers = LG_REALTIME_DOT | (generation ? 0 : LG_CACHE_GRID) | ((redraw_graph || !generation) ? LG_CACHE_GRAPH : 0);
     return true;
@@ -291,7 +291,8 @@ gain_reduction2_audio_module::gain_reduction2_audio_module()
     mute            = -1;
     old_y1          = 0.f;
     old_yl          = 0.f;
-    old_detected    = 0.f;
+    old_mae         = 0.f;
+    old_mre         = 0.f;
     redraw_graph    = true;
 }
 
@@ -342,18 +343,24 @@ void gain_reduction2_audio_module::process(float &left)
             
         xl = xg - yg;
             
-        y1 = std::max(xl, release_coeff*old_y1+(1.f-release_coeff)*xl);
-        yl = attack_coeff*old_yl+(1.f-attack_coeff)*y1;
+        y1 = _sanitize(std::max(xl, release_coeff*old_y1+(1.f-release_coeff)*xl));
+        yl = _sanitize(attack_coeff*old_yl+(1.f-attack_coeff)*y1);
         
         cdb = -yl;
         gain = exp(cdb/20.f*log(10.f));
 
-    left *= gain * makeup;
-    meter_out = (fabs(left));
+        left *= gain * makeup;
+        meter_out = (fabs(left));
         meter_comp = gain;
-    detected = (exp(yg/20.f*log(10.f))+old_detected)/2.f;
-    old_detected = detected;
 
+        float mre = _sanitize(std::max(xg, release_coeff*old_mre+(1.f-release_coeff)*xg));
+        float mae = _sanitize(attack_coeff*old_mae+(1.f-attack_coeff)*mre);
+        
+        old_mre = mre;
+        old_mae = mae;
+        
+        detected = exp(mae/20.f*log(10.f));
+    
         old_yl = yl;
         old_y1 = y1;
     }
@@ -423,7 +430,7 @@ float gain_reduction2_audio_module::get_comp_level() {
     return meter_comp;
 }
 
-bool gain_reduction2_audio_module::get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const
+bool gain_reduction2_audio_module::_get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const
 {
     redraw_graph = false;
     if (!is_active or subindex > 1)
@@ -452,7 +459,7 @@ bool gain_reduction2_audio_module::get_graph(int subindex, float *data, int poin
     return true;
 }
 
-bool gain_reduction2_audio_module::get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const
+bool gain_reduction2_audio_module::_get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const
 {
     if (!is_active or bypass > 0.5f or mute > 0.f or subindex)
         return false;
@@ -464,7 +471,7 @@ bool gain_reduction2_audio_module::get_dot(int subindex, float &x, float &y, int
     return true;
 }
 
-bool gain_reduction2_audio_module::get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+bool gain_reduction2_audio_module::_get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
 {
     bool tmp;
     vertical = (subindex & 1) != 0;
@@ -483,7 +490,7 @@ bool gain_reduction2_audio_module::get_gridline(int subindex, float &pos, bool &
     return result;
 }
 
-bool gain_reduction2_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+bool gain_reduction2_audio_module::_get_layers(int index, int generation, unsigned int &layers) const
 {
     layers = LG_REALTIME_DOT | (generation ? 0 : LG_CACHE_GRID) | ((redraw_graph || !generation) ? LG_CACHE_GRAPH : 0);
     return true;
@@ -658,7 +665,7 @@ float expander_audio_module::get_expander_level() {
     return meter_gate;
 }
 
-bool expander_audio_module::get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const
+bool expander_audio_module::_get_graph(int subindex, float *data, int points, cairo_iface *context, int *mode) const
 {
     redraw_graph = false;
     if (!is_active or subindex > 1)
@@ -686,7 +693,7 @@ bool expander_audio_module::get_graph(int subindex, float *data, int points, cai
     return true;
 }
 
-bool expander_audio_module::get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const
+bool expander_audio_module::_get_dot(int subindex, float &x, float &y, int &size, cairo_iface *context) const
 {
     if (!is_active or bypass > 0.5f or mute > 0.f or subindex)
         return false;
@@ -698,7 +705,7 @@ bool expander_audio_module::get_dot(int subindex, float &x, float &y, int &size,
     return true;
 }
 
-bool expander_audio_module::get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
+bool expander_audio_module::_get_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
 {
     bool tmp;
     vertical = (subindex & 1) != 0;
@@ -717,7 +724,7 @@ bool expander_audio_module::get_gridline(int subindex, float &pos, bool &vertica
     return result;
 }
 
-bool expander_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+bool expander_audio_module::_get_layers(int index, int generation, unsigned int &layers) const
 {
     layers = LG_REALTIME_DOT | (generation ? 0 : LG_CACHE_GRID) | ((redraw_graph || !generation) ? LG_CACHE_GRAPH : 0);
     return true;
@@ -823,22 +830,22 @@ uint32_t compressor_audio_module::process(uint32_t offset, uint32_t numsamples,
 }
 bool compressor_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
 {
-    return compressor.get_graph(subindex, data, points, context, mode);
+    return compressor._get_graph(subindex, data, points, context, mode);
 }
 
 bool compressor_audio_module::get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const
 {
-    return compressor.get_dot(subindex, x, y, size, context);
+    return compressor._get_dot(subindex, x, y, size, context);
 }
 
 bool compressor_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
 {
-    return compressor.get_gridline(subindex, pos, vertical, legend, context);
+    return compressor._get_gridline(subindex, pos, vertical, legend, context);
 }
 
 bool compressor_audio_module::get_layers(int index, int generation, unsigned int &layers) const
 {
-    return compressor.get_layers(index, generation, layers);
+    return compressor._get_layers(index, generation, layers);
 }
 
 /**********************************************************************
@@ -1075,21 +1082,20 @@ uint32_t sidechaincompressor_audio_module::process(uint32_t offset, uint32_t num
 
             float leftAC  = inL;
             float rightAC = inR;
-            float leftSC  = inL;
-            float rightSC = inR;
+            float leftSC  = inL * *params[param_sc_level];
+            float rightSC = inR * *params[param_sc_level];
             float leftMC  = inL;
             float rightMC = inR;
+            float leftTC  = inL;
+            float rightTC = inR;
             
             if (*params[param_sc_route] > 0.5) {
-                leftAC  = inL;
-                rightAC = inR;
                 leftSC  = in2L * *params[param_sc_level];
                 rightSC = in2R * *params[param_sc_level];
-                leftMC  = inL;
-                rightMC = inR;
             }
             
-            switch ((CalfScModes)*params[param_sc_mode]) {
+            int mode = (CalfScModes)*params[param_sc_mode];
+            switch (mode) {
                 default:
                 case WIDEBAND:
                     compressor.process(leftAC, rightAC, &leftSC, &rightSC);
@@ -1108,28 +1114,6 @@ uint32_t sidechaincompressor_audio_module::process(uint32_t offset, uint32_t num
                     rightMC = rightSC;
                     compressor.process(leftAC, rightAC, &leftSC, &rightSC);
                     break;
-                case DEESSER_SPLIT:
-                    leftSC  = f2L.process(leftSC);
-                    rightSC = f2R.process(rightSC);
-                    leftMC  = leftSC;
-                    rightMC = rightSC;
-                    compressor.process(leftSC, rightSC, &leftSC, &rightSC);
-                    leftAC   = f1L.process(leftAC);
-                    rightAC  = f1R.process(rightAC);
-                    leftAC  += leftSC;
-                    rightAC += rightSC;
-                    break;
-                case DERUMBLER_SPLIT:
-                    leftSC  = f1L.process(leftSC);
-                    rightSC = f1R.process(rightSC);
-                    leftMC  = leftSC;
-                    rightMC = rightSC;
-                    compressor.process(leftSC, rightSC, &leftSC, &rightSC);
-                    leftAC   = f2L.process(leftAC);
-                    rightAC  = f2R.process(rightAC);
-                    leftAC  += leftSC;
-                    rightAC += rightSC;
-                    break;
                 case BANDPASS_1:
                     leftSC  = f1L.process(leftSC);
                     rightSC = f1R.process(rightSC);
@@ -1137,6 +1121,29 @@ uint32_t sidechaincompressor_audio_module::process(uint32_t offset, uint32_t num
                     rightMC = rightSC;
                     compressor.process(leftAC, rightAC, &leftSC, &rightSC);
                     break;
+                case DEESSER_SPLIT:
+                case DERUMBLER_SPLIT:
+                    if (mode == DEESSER_SPLIT) {
+                        leftTC  = f2L.process(leftTC);
+                        rightTC = f2R.process(rightTC);
+                        leftAC   = f1L.process(leftAC);
+                        rightAC  = f1R.process(rightAC);
+                    } else {
+                        leftTC  = f1L.process(leftTC);
+                        rightTC = f1R.process(rightTC);
+                        leftAC   = f2L.process(leftAC);
+                        rightAC  = f2R.process(rightAC);
+                    }
+                    if (*params[param_sc_route] <= 0.5) {
+                        leftSC  = leftTC;
+                        rightSC = rightTC;
+                    }
+                    compressor.process(leftTC, rightTC, &leftSC, &rightSC);
+                    leftMC  = leftSC;
+                    rightMC = rightSC;
+                    leftAC  += leftTC;
+                    rightAC += rightTC;
+                    break;
             }
 
             if(*params[param_sc_listen] > 0.f) {
@@ -1176,7 +1183,7 @@ bool sidechaincompressor_audio_module::get_graph(int index, int subindex, int ph
     if (index == param_sc_listen && !subindex) {
         return ::get_graph(*this, subindex, data, points);
     } else if(index == param_bypass) {
-        return compressor.get_graph(subindex, data, points, context, mode);
+        return compressor._get_graph(subindex, data, points, context, mode);
     }
     return false;
 }
@@ -1186,7 +1193,7 @@ bool sidechaincompressor_audio_module::get_dot(int index, int subindex, int phas
     if (!is_active or !phase)
         return false;
     if (index == param_bypass) {
-        return compressor.get_dot(subindex, x, y, size, context);
+        return compressor._get_dot(subindex, x, y, size, context);
     }
     return false;
 }
@@ -1196,7 +1203,7 @@ bool sidechaincompressor_audio_module::get_gridline(int index, int subindex, int
     if (!is_active or phase)
         return false;
     if (index == param_bypass) {
-        return compressor.get_gridline(subindex, pos, vertical, legend, context);
+        return compressor._get_gridline(subindex, pos, vertical, legend, context);
     } else {
         return get_freq_gridline(subindex, pos, vertical, legend, context);
     }
@@ -1206,7 +1213,7 @@ bool sidechaincompressor_audio_module::get_gridline(int index, int subindex, int
 bool sidechaincompressor_audio_module::get_layers(int index, int generation, unsigned int &layers) const
 {
     if(index == param_bypass)
-        return compressor.get_layers(index, generation, layers);
+        return compressor._get_layers(index, generation, layers);
     bool redraw = redraw_graph || !generation;
     layers = 0 | (generation ? 0 : LG_CACHE_GRID) | (redraw ? LG_CACHE_GRAPH : 0);
     redraw_graph = false;
@@ -1406,7 +1413,7 @@ bool multibandcompressor_audio_module::get_graph(int index, int subindex, int ph
         
     const gain_reduction_audio_module *m = get_strip_by_param_index(index);
     if (m) {
-        r = m->get_graph(subindex, data, points, context, mode);
+        r = m->_get_graph(subindex, data, points, context, mode);
     } else {
         r = crossover.get_graph(subindex, phase, data, points, context, mode);
     }
@@ -1430,7 +1437,7 @@ bool multibandcompressor_audio_module::get_dot(int index, int subindex, int phas
 {
     const gain_reduction_audio_module *m = get_strip_by_param_index(index);
     if (m)
-        return m->get_dot(subindex, x, y, size, context);
+        return m->_get_dot(subindex, x, y, size, context);
     return false;
 }
 
@@ -1438,7 +1445,7 @@ bool multibandcompressor_audio_module::get_gridline(int index, int subindex, int
 {
     const gain_reduction_audio_module *m = get_strip_by_param_index(index);
     if (m)
-        return m->get_gridline(subindex, pos, vertical, legend, context);
+        return m->_get_gridline(subindex, pos, vertical, legend, context);
     if (phase) return false;
     return get_freq_gridline(subindex, pos, vertical, legend, context);
 }
@@ -1448,7 +1455,7 @@ bool multibandcompressor_audio_module::get_layers(int index, int generation, uns
     bool r;
     const gain_reduction_audio_module *m = get_strip_by_param_index(index);
     if (m) {
-        r = m->get_layers(index, generation, layers);
+        r = m->_get_layers(index, generation, layers);
     } else {
         r = crossover.get_layers(index, generation, layers);
     }
@@ -1557,22 +1564,22 @@ uint32_t monocompressor_audio_module::process(uint32_t offset, uint32_t numsampl
 }
 bool monocompressor_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
 {
-    return monocompressor.get_graph(subindex, data, points, context, mode);
+    return monocompressor._get_graph(subindex, data, points, context, mode);
 }
 
 bool monocompressor_audio_module::get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const
 {
-    return monocompressor.get_dot(subindex, x, y, size, context);
+    return monocompressor._get_dot(subindex, x, y, size, context);
 }
 
 bool monocompressor_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
 {
-    return monocompressor.get_gridline(subindex, pos, vertical, legend, context);
+    return monocompressor._get_gridline(subindex, pos, vertical, legend, context);
 }
 
 bool monocompressor_audio_module::get_layers(int index, int generation, unsigned int &layers) const
 {
-    return monocompressor.get_layers(index, generation, layers);
+    return monocompressor._get_layers(index, generation, layers);
 }
 
 /**********************************************************************
@@ -1662,8 +1669,10 @@ uint32_t deesser_audio_module::process(uint32_t offset, uint32_t numsamples, uin
 {
     bool bypassed = bypass.update(*params[param_bypass] > 0.5f, numsamples);
     numsamples += offset;
+    detected_led -= std::min(detected_led,  numsamples);
+    float gain = 1.f;
     if(bypassed) {
-        // everything bypassed81e8da266
+        // everything bypassed
         while(offset < numsamples) {
             outs[0][offset] = ins[0][offset];
             outs[1][offset] = ins[1][offset];
@@ -1675,7 +1684,6 @@ uint32_t deesser_audio_module::process(uint32_t offset, uint32_t numsamples, uin
         // process
         uint32_t orig_numsamples = numsamples-offset;
         uint32_t orig_offset = offset;
-        detected_led -= std::min(detected_led,  numsamples);
         compressor.update_curve();
 
         while(offset < numsamples) {
@@ -1730,14 +1738,12 @@ uint32_t deesser_audio_module::process(uint32_t offset, uint32_t numsamples, uin
             outs[0][offset] = outL;
             outs[1][offset] = outR;
 
-            if(std::max(fabs(leftSC), fabs(rightSC)) > *params[param_threshold]) {
-                detected_led   = srate >> 3;
-            }
             detected = std::max(fabs(leftMC), fabs(rightMC));
             
-            float values[] = {detected, compressor.get_comp_level()};
+            float comp = compressor.get_comp_level();
+            float values[] = {detected, comp};
             meters.process(values);
-            
+            gain = std::min(comp, gain);
             // next sample
             ++offset;
         } // cycle trough samples
@@ -1749,10 +1755,9 @@ uint32_t deesser_audio_module::process(uint32_t offset, uint32_t numsamples, uin
         pL.sanitize();
         pR.sanitize();
     }
-    // draw meters
-    if(params[param_detected_led] != NULL) {
-        *params[param_detected_led] = detected_led;
-    }
+    if(params[param_detected_led] != NULL and gain < 0.89)
+        detected_led = srate >> 3;
+    *params[param_detected_led] = detected_led;
     meters.fall(numsamples);
     return outputs_mask;
 }
@@ -1848,22 +1853,22 @@ uint32_t gate_audio_module::process(uint32_t offset, uint32_t numsamples, uint32
 }
 bool gate_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
 {
-    return gate.get_graph(subindex, data, points, context, mode);
+    return gate._get_graph(subindex, data, points, context, mode);
 }
 
 bool gate_audio_module::get_dot(int index, int subindex, int phase, float &x, float &y, int &size, cairo_iface *context) const
 {
-    return gate.get_dot(subindex, x, y, size, context);
+    return gate._get_dot(subindex, x, y, size, context);
 }
 
 bool gate_audio_module::get_gridline(int index, int subindex, int phase, float &pos, bool &vertical, std::string &legend, cairo_iface *context) const
 {
-    return gate.get_gridline(subindex, pos, vertical, legend, context);
+    return gate._get_gridline(subindex, pos, vertical, legend, context);
 }
 
 bool gate_audio_module::get_layers(int index, int generation, unsigned int &layers) const
 {
-    return gate.get_layers(index, generation, layers);
+    return gate._get_layers(index, generation, layers);
 }
 
 /**********************************************************************
@@ -2092,21 +2097,21 @@ uint32_t sidechaingate_audio_module::process(uint32_t offset, uint32_t numsample
             
             float leftAC  = inL;
             float rightAC = inR;
-            float leftSC  = inL;
-            float rightSC = inR;
+            float leftSC  = inL * *params[param_sc_level];
+            float rightSC = inR * *params[param_sc_level];
             float leftMC  = inL;
             float rightMC = inR;
+            float leftTC  = inL;
+            float rightTC = inR;
             
             if (*params[param_sc_route] > 0.5) {
-                leftAC  = inL;
-                rightAC = inR;
                 leftSC  = in2L * *params[param_sc_level];
                 rightSC = in2R * *params[param_sc_level];
-                leftMC  = inL;
-                rightMC = inR;
             }
             
-            switch ((CalfScModes)*params[param_sc_mode]) {
+            int mode = (CalfScModes)*params[param_sc_mode];
+            
+            switch (mode) {
                 default:
                 case WIDEBAND:
                     gate.process(leftAC, rightAC, &leftSC, &rightSC);
@@ -2125,28 +2130,6 @@ uint32_t sidechaingate_audio_module::process(uint32_t offset, uint32_t numsample
                     rightMC = rightSC;
                     gate.process(leftAC, rightAC, &leftSC, &rightSC);
                     break;
-                case HIGHGATE_SPLIT:
-                    leftSC  = f2L.process(leftSC);
-                    rightSC = f2R.process(rightSC);
-                    leftMC  = leftSC;
-                    rightMC = rightSC;
-                    gate.process(leftSC, rightSC, &leftSC, &rightSC);
-                    leftAC   = f1L.process(leftAC);
-                    rightAC  = f1R.process(rightAC);
-                    leftAC  += leftSC;
-                    rightAC += rightSC;
-                    break;
-                case LOWGATE_SPLIT:
-                    leftSC  = f1L.process(leftSC);
-                    rightSC = f1R.process(rightSC);
-                    leftMC  = leftSC;
-                    rightMC = rightSC;
-                    gate.process(leftSC, rightSC, &leftSC, &rightSC);
-                    leftAC   = f2L.process(leftAC);
-                    rightAC  = f2R.process(rightAC);
-                    leftAC  += leftSC;
-                    rightAC += rightSC;
-                    break;
                 case BANDPASS_1:
                     leftSC  = f1L.process(leftSC);
                     rightSC = f1R.process(rightSC);
@@ -2154,6 +2137,29 @@ uint32_t sidechaingate_audio_module::process(uint32_t offset, uint32_t numsample
                     rightMC = rightSC;
                     gate.process(leftAC, rightAC, &leftSC, &rightSC);
                     break;
+                case HIGHGATE_SPLIT:
+                case LOWGATE_SPLIT:
+                    if (mode == HIGHGATE_SPLIT) {
+                        leftTC  = f2L.process(leftTC);
+                        rightTC = f2R.process(rightTC);
+                        leftAC   = f1L.process(leftAC);
+                        rightAC  = f1R.process(rightAC);
+                    } else {
+                        leftTC  = f1L.process(leftTC);
+                        rightTC = f1R.process(rightTC);
+                        leftAC   = f2L.process(leftAC);
+                        rightAC  = f2R.process(rightAC);
+                    }
+                    if (*params[param_sc_route] <= 0.5) {
+                        leftSC  = leftTC;
+                        rightSC = rightTC;
+                    }
+                    gate.process(leftTC, rightTC, &leftSC, &rightSC);
+                    leftMC  = leftSC;
+                    rightMC = rightSC;
+                    leftAC  += leftTC;
+                    rightAC += rightTC;
+                    break;
             }
 
             if(*params[param_sc_listen] > 0.f) {
@@ -2191,7 +2197,7 @@ bool sidechaingate_audio_module::get_graph(int index, int subindex, int phase, f
     if (index == param_sc_listen && !subindex) {
         return ::get_graph(*this, subindex, data, points);
     } else if(index == param_bypass) {
-        return gate.get_graph(subindex, data, points, context, mode);
+        return gate._get_graph(subindex, data, points, context, mode);
     }
     return false;
 }
@@ -2201,7 +2207,7 @@ bool sidechaingate_audio_module::get_dot(int index, int subindex, int phase, flo
     if (!is_active or !phase)
         return false;
     if (index == param_bypass) {
-        return gate.get_dot(subindex, x, y, size, context);
+        return gate._get_dot(subindex, x, y, size, context);
     }
     return false;
 }
@@ -2211,7 +2217,7 @@ bool sidechaingate_audio_module::get_gridline(int index, int subindex, int phase
     if (!is_active or phase)
         return false;
     if (index == param_bypass) {
-        return gate.get_gridline(subindex, pos, vertical, legend, context);
+        return gate._get_gridline(subindex, pos, vertical, legend, context);
     } else {
         return get_freq_gridline(subindex, pos, vertical, legend, context);
     }
@@ -2220,7 +2226,7 @@ bool sidechaingate_audio_module::get_gridline(int index, int subindex, int phase
 bool sidechaingate_audio_module::get_layers(int index, int generation, unsigned int &layers) const
 {
     if(index == param_bypass)
-        return gate.get_layers(index, generation, layers);
+        return gate._get_layers(index, generation, layers);
     bool redraw = redraw_graph || !generation;
     layers = 0 | (generation ? 0 : LG_CACHE_GRID) | (redraw ? LG_CACHE_GRAPH : 0);
     redraw_graph = false;
@@ -2418,7 +2424,7 @@ bool multibandgate_audio_module::get_graph(int index, int subindex, int phase, f
         
     const expander_audio_module *m = get_strip_by_param_index(index);
     if (m) {
-        r = m->get_graph(subindex, data, points, context, mode);
+        r = m->_get_graph(subindex, data, points, context, mode);
     } else {
         r = crossover.get_graph(subindex, phase, data, points, context, mode);
     }
@@ -2442,7 +2448,7 @@ bool multibandgate_audio_module::get_dot(int index, int subindex, int phase, flo
 {
     const expander_audio_module *m = get_strip_by_param_index(index);
     if (m)
-        return m->get_dot(subindex, x, y, size, context);
+        return m->_get_dot(subindex, x, y, size, context);
     return false;
 }
 
@@ -2450,7 +2456,7 @@ bool multibandgate_audio_module::get_gridline(int index, int subindex, int phase
 {
     const expander_audio_module *m = get_strip_by_param_index(index);
     if (m)
-        return m->get_gridline(subindex, pos, vertical, legend, context);
+        return m->_get_gridline(subindex, pos, vertical, legend, context);
     if (phase) return false;
     return get_freq_gridline(subindex, pos, vertical, legend, context);
 }
@@ -2460,7 +2466,7 @@ bool multibandgate_audio_module::get_layers(int index, int generation, unsigned
     bool r;
     const expander_audio_module *m = get_strip_by_param_index(index);
     if (m) {
-        r = m->get_layers(index, generation, layers);
+        r = m->_get_layers(index, generation, layers);
     } else {
         r = crossover.get_layers(index, generation, layers);
     }
@@ -2492,6 +2498,7 @@ transientdesigner_audio_module::transientdesigner_audio_module() {
     transients.set_channels(channels);
     hp_f_old = hp_m_old = lp_f_old = lp_m_old = 0;
     redraw = false;
+    pbuffer = NULL;
 }
 transientdesigner_audio_module::~transientdesigner_audio_module()
 {
