@@ -22,6 +22,7 @@
 #include <memory.h>
 #include <calf/giface.h>
 #include <calf/modules_dist.h>
+#include <string>
 
 using namespace dsp;
 using namespace calf_plugins;
@@ -172,19 +173,21 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
             
             for (int i = 0; i < c; ++i) {
                 // all pre filters in chain
-                proc[i] = lp[i][1].process(lp[i][0].process(proc[i]));
-                proc[i] = hp[i][1].process(hp[i][0].process(proc[i]));
-
+                if (*params[param_pre] > 0.5) {
+                    proc[i] = lp[i][1].process(lp[i][0].process(proc[i]));
+                    proc[i] = hp[i][1].process(hp[i][0].process(proc[i]));
+                }
                 // ...saturate...
                 proc[i] = dist[i].process(proc[i]);
                 
                 // tone control
                 proc[i] = p[i].process(proc[i]);
-
-                // all post filters in chain
-                proc[i] = lp[i][2].process(lp[i][3].process(proc[i]));
-                proc[i] = hp[i][2].process(hp[i][3].process(proc[i]));
                 
+                // all post filters in chain
+                if (*params[param_post] > 0.5) {
+                    proc[i] = lp[i][2].process(lp[i][3].process(proc[i]));
+                    proc[i] = hp[i][2].process(hp[i][3].process(proc[i]));
+                }
                 //subtract gain
                 proc[i] *= onedivlevelin;
                 
@@ -626,6 +629,233 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
     return outputs_mask;
 }
 
+
+/**********************************************************************
+ * VINYL by Markus Schmidt
+**********************************************************************/
+
+vinyl_audio_module::vinyl_audio_module() {
+    active          = false;
+    clip_inL        = 0.f;
+    clip_inR        = 0.f;
+    clip_outL       = 0.f;
+    clip_outR       = 0.f;
+    meter_inL       = 0.f;
+    meter_inR       = 0.f;
+    meter_outL      = 0.f;
+    meter_outR      = 0.f;
+    dbufsize        = 0;
+    
+    speed_old       = 0.f;
+    freq_old        = 0.f;
+    aging_old       = 0.f;
+}
+
+void vinyl_audio_module::activate() {
+    active = true;
+}
+
+void vinyl_audio_module::deactivate() {
+    active = false;
+}
+
+void vinyl_audio_module::params_changed() {
+    if (speed_old != *params[param_speed]) {
+        lfo.set_params(1.f / (60.f / *params[param_speed]), 0, 0.f, srate, 0.5f);
+        speed_old = *params[param_speed];
+    }
+    if (freq_old != *params[param_freq] or aging_old != *params[param_aging]) {
+        aging_old = *params[param_aging];
+        freq_old = *params[param_freq];
+        float lp = (freq_old + 500.f) * pow(double(20000.f / (freq_old + 500.f)), 1.f - aging_old);
+        float hp = 10.f * pow(double((freq_old - 250) / 10.f), aging_old);
+        filters[0][0].set_hp_rbj(hp, 0.707 + aging_old * 0.5, (float)srate);
+        filters[0][1].copy_coeffs(filters[0][0]);
+        filters[0][2].set_peakeq_rbj(freq_old, 1, 1.f + aging_old * 4.f, (float)srate);
+        filters[0][3].set_lp_rbj(lp, 0.707 + aging_old * 0.5, (float)srate);
+        filters[0][4].copy_coeffs(filters[0][0]);
+        for (int i = 1; i < channels; i++) {
+            for (int j = 0; j < _filters; j++)
+                filters[i][j].copy_coeffs(filters[0][j]);
+        }
+    }
+    for (int j = 0; j < _synths; j++) {
+        fluid_synth_pitch_bend(synth, j, (int)(*params[param_pitch0 + j * _synthsp] * 8191 + 8192));
+    }
+}
+
+uint32_t vinyl_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask) {
+    bool bypassed = bypass.update(*params[param_bypass] > 0.5f, numsamples);
+    uint32_t orig_offset = offset;
+    float sL[numsamples], sR[numsamples];
+    if (!bypassed) {
+        for (int j = 0; j < _synths; ++j) {
+            float gain = 0;
+            if (*params[param_active0 + j * _synthsp] >= 0.5f) {
+                gain = *params[param_gain0 + j * _synthsp];
+            }
+            if (gain == last_gain[j])
+                continue;
+            if (last_gain[j] == 0 && gain != 0) {
+                fluid_synth_noteon(synth, j, 60, 127);
+            }
+            if (last_gain[j] != 0 && gain == 0) {
+                fluid_synth_noteoff(synth, j, 60);
+            }
+            if (gain > 0) {
+                fluid_synth_set_gen(synth, j, GEN_ATTENUATION, 200 * log10(1.0 / gain));
+            }
+            last_gain[j] = gain;
+        }
+        fluid_synth_write_float(synth, numsamples, sL, 0, 1, sR, 0, 1);
+    }
+    for(uint32_t i = offset; i < offset + numsamples; i++) {
+        float L = ins[0][i];
+        float R = ins[1][i];
+        if(bypassed) {
+            outs[0][i]  = ins[0][i];
+            outs[1][i]  = ins[1][i];
+            float values[] = {0, 0, 0, 0};
+            meters.process(values);
+        } else {
+            // gain
+            L *= *params[param_level_in];
+            R *= *params[param_level_in];
+            
+            // meters
+            float inL = L;
+            float inR = R;
+            
+            // droning
+            dbuf[dbufpos * channels + 0] = L;
+            dbuf[dbufpos * channels + 1] = R;
+            
+            if (*params[param_drone] > 0.f) {
+                int32_t bpos = (lfo.get_value() + 0.5) * *params[param_drone] * dbufrange;
+                bpos = (dbufpos - bpos) & (dbufsize - 1);
+            
+                L = dbuf[bpos * channels];
+                R = dbuf[bpos * channels + 1];
+
+                lfo.advance(1);
+            }
+            dbufpos = (dbufpos + 1) & (dbufsize - 1);
+            
+            // synths
+            L += sL[i - offset];
+            R += sR[i - offset];
+            
+            // filter
+            if (*params[param_aging] > 0.f) {
+                for (int j = 0; j < _filters; j++) {
+                    L = filters[0][j].process(L);
+                    R = filters[1][j].process(R);
+                }
+            }
+            
+            // levels out
+            float areduce = (1.f - aging_old * 0.9);
+            L *= *params[param_level_out] * areduce;
+            R *= *params[param_level_out] * areduce;
+            
+            // output
+            outs[0][i] = L;
+            outs[1][i] = R;
+            
+            // sanitize filters
+            if (*params[param_aging] > 0.f) {
+                for (int j = 0; j < _filters; j++) {
+                    filters[0][j].sanitize();
+                    filters[1][j].sanitize();
+                }
+            }
+            
+            float values[] = {inL, inR, outs[0][i], outs[1][i]};
+            meters.process(values);
+        }
+    }
+    if (bypassed)
+        bypass.crossfade(ins, outs, 2, orig_offset, numsamples);
+    meters.fall(numsamples);
+    return outputs_mask;
+}
+
+void vinyl_audio_module::set_sample_rate(uint32_t sr)
+{
+    srate = sr;
+    int meter[] = {param_meter_inL,  param_meter_inR, param_meter_outL, param_meter_outR};
+    int clip[]  = {param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR};
+    meters.init(params, meter, clip, 4, srate);
+}
+
+void vinyl_audio_module::post_instantiate(uint32_t sr)
+{    
+    dbufsize = (sr + 49) / 50;
+    // Round up to the nearest power of 2 if it's not already
+    // This makes the circular buffer implementation faster and simpler
+    if((dbufsize & (dbufsize - 1)))
+        dbufsize = 1 << (32 - __builtin_clz(dbufsize - 1));
+    dbufrange = sr / 100.0;
+    dbuf = (float*) calloc(dbufsize * channels, sizeof(float));
+    dbufpos = 0;
+    settings = new_fluid_settings();
+    fluid_settings_setnum(settings, "synth.sample-rate", sr);
+    fluid_settings_setint(settings, "synth.polyphony", 32);
+    fluid_settings_setint(settings, "synth.midi-channels", _synths);
+    fluid_settings_setint(settings, "synth.reverb.active", 0);
+    fluid_settings_setint(settings, "synth.chorus.active", 0);
+    
+    char const * paths[] = {
+        PKGLIBDIR "sf2/Hum.sf2",
+        PKGLIBDIR "sf2/Motor.sf2",
+        PKGLIBDIR "sf2/Static.sf2",
+        PKGLIBDIR "sf2/Noise.sf2",
+        PKGLIBDIR "sf2/Rumble.sf2",
+        PKGLIBDIR "sf2/Crackle.sf2",
+        PKGLIBDIR "sf2/Crinkle.sf2"
+    };
+    synth = new_fluid_synth(settings);
+    fluid_synth_set_gain(synth, 1.f);
+    for (int i = 0; i < _synths; i++) {
+        int id = fluid_synth_sfload(synth, paths[i], 0);
+        fluid_synth_program_select (synth, i, id, 0, 0);
+        fluid_synth_pitch_wheel_sens(synth, i, 12);
+        last_gain[i] = 0;
+    }
+}
+vinyl_audio_module::~vinyl_audio_module()
+{
+    free(dbuf);
+    delete_fluid_synth(synth);
+    delete_fluid_settings(settings);
+}
+
+
+bool vinyl_audio_module::get_graph(int index, int subindex, int phase, float *data, int points, cairo_iface *context, int *mode) const
+{
+    if (subindex > 0)
+        return false;
+    return ::get_graph(*this, subindex, data, points);
+}
+float vinyl_audio_module::freq_gain(int index, double freq) const
+{
+    float s = 1.f;
+    if (*params[param_aging] > 0)
+        for (int i = 0; i < _filters; i++)
+            s *= filters[0][i].freq_gain(freq, srate);
+    return s;
+}
+bool vinyl_audio_module::get_layers(int index, int generation, unsigned int &layers) const
+{
+    layers = 0;
+    if (!generation)
+        layers |= LG_CACHE_GRID;
+    if (index == param_freq)
+        layers |= LG_REALTIME_GRAPH;
+    return true;
+}
+
+
 /**********************************************************************
  * TAPESIMULATOR by Markus Schmidt
 **********************************************************************/
@@ -794,8 +1024,8 @@ uint32_t tapesimulator_audio_module::process(uint32_t offset, uint32_t numsample
             lfo2.advance(1);
             
             // dot
-            rms = std::max((double)rms, (fabs(Lo) + fabs(Ro)) / 2);
-            input = std::max((double)input, (fabs(Lc) + fabs(Rc)) / 2);
+            rms = std::max((double)rms, (double)((fabs(Lo) + fabs(Ro)) / 2));
+            input = std::max((double)input, (double)((fabs(Lc) + fabs(Rc)) / 2));
             
             float values[] = {inL, inR, outs[0][i], outs[1][i]};
             meters.process(values);
